import { world, system, BlockPermutation} from '@minecraft/server';
let brushProp = {color: "black", size: "Small", tool: "None"};
let playerUseItem, location, selection, item, playerPastLocation;
let outlineDelay = true;
let selectionList = [];
let universalCheck = false;
let exit = false;
world.getDimension(`overworld`).runCommand(`tickingarea add 4928 319 64 5056 275 -65 painting_room true`);
world.getDimension(`overworld`).runCommand(`gamerule sendcommandfeedback false`);
world.getDimension(`overworld`).runCommand(`gamerule commandblockoutput false`);
function checkDrawer() {
    for (const player of world.getAllPlayers()) {
        const riding = player.getComponent( "minecraft:riding" )?.entityRidingOn;
        const inventory = player.getComponent( "minecraft:inventory" ).container;
        if (
            riding?.typeId == "ae:painting_stand"
            && inventory.getItem(0)?.typeId == "minecraft:brush"
            && inventory.getItem(1)?.typeId == "minecraft:empty_map"
        ) {
            playerPastLocation = player.location;
            if (!player.hasTag( "drawer" )) player.addTag( "drawer" );
            world.getDimension(`overworld`).runCommand(`structure load mystructure:painting_room_v2 4976 277 -16 0_degrees none true true`);
            world.getDimension(`overworld`).runCommand(`execute as @a[tag=drawer] run gamemode adventure @s`);
            world.getDimension("overworld").fillBlocks({x:4928, y:319, z:64}, {x:5056, y:319, z:-65}, BlockPermutation.resolve("minecraft:white_wool"));
            player.addEffect('night_vision', 9999999, { showParticles: false });
            player.teleport({x:4999.509, y:295, z:12.425});
            let blockCheckExist = world.getDimension("overworld").getBlock({x: 5002, y: 300, z: 14});
            if (playerPastLocation && blockCheckExist) {
                world.getDimension("overworld").getBlock({x: 5002, y: 302, z: 14}).getComponent("minecraft:sign")?.setText(playerPastLocation.x.toString());
                world.getDimension("overworld").getBlock({x: 5001, y: 302, z: 14}).getComponent("minecraft:sign")?.setText(playerPastLocation.y.toString());
                world.getDimension("overworld").getBlock({x: 5000, y: 302, z: 14}).getComponent("minecraft:sign")?.setText(playerPastLocation.z.toString());
            } else {
                console.warn("Something has gone Wrong");
            }
            
            universalCheck = true;
        };
    };
};
async function removeAll() {
    world.getDimension("overworld").fillBlocks({x:4928, y:319, z:64}, {x:5056, y:319, z:-65}, BlockPermutation.resolve("minecraft:air"));
    world.getDimension("overworld").fillBlocks({x:4976, y:318, z:15}, {x:4991, y:277, z:-16}, BlockPermutation.resolve("minecraft:air"));
    world.getDimension("overworld").fillBlocks({x:4992, y:318, z:15}, {x:5007, y:277, z:-16}, BlockPermutation.resolve("minecraft:air"));
    world.getDimension("overworld").fillBlocks({x:5008, y:277, z:-16}, {x:5022, y:318, z:15}, BlockPermutation.resolve("minecraft:air"));
}
function render() {
    for (let xDirect = 4983; xDirect <= 5014; xDirect++) {
        let mapX=4928 - 4*(4983- xDirect);
        for (let yDirect = 311; yDirect >= 280; yDirect--) {
            let mapZ=-64 - 4*(yDirect-311);
            let pixel = world.getDimension(`overworld`).getBlock({x: xDirect, y: yDirect, z: -14})?.type;
            if (pixel) {world.getDimension(`overworld`).fillBlocks({x:mapX, y:319, z:mapZ}, {x:mapX+4, y:319, z:mapZ+4}, pixel)};
        }
    };
};
function clamp(value, min, max) {
    return Math.min(Math.max(value, min), max);
}
function updateFillTool(color) {
    world.getDimension("overworld").fillBlocks({x:5011, y:278, z:-9}, {x:5015, y:278, z:-5}, BlockPermutation.resolve(`minecraft:${color}_shulker_box`));
}
function fakeInput(location) {
    if (brushProp.size === "Small") 
    {
        playerUseItem.runCommandAsync(`fill ${location?.x} ${location?.y} -14 ${location?.x} ${location?.y} -14 minecraft:${brushProp.color}_wool replace wool`);
    } 
    else if (brushProp.size === "Medium") 
    {
        playerUseItem.runCommandAsync(`fill ${location?.x} ${location?.y} -14 ${location?.x+1} ${location?.y+1} -14 minecraft:${brushProp.color}_wool replace wool`);
    }
    else if (brushProp.size === "Big") {
        playerUseItem.runCommandAsync(`fill ${location.x-1} ${location.y-1} -14 ${location.x+1} ${location.y+1} -14 minecraft:${brushProp.color}_wool replace wool`)
    }
    else if (brushProp.size === "Huge"){
        playerUseItem.runCommandAsync(`fill ${location.x-2} ${location.y-2} -14 ${location.x+2} ${location.y+2} -14 minecraft:${brushProp.color}_wool replace wool`)
    };
};
function toolPreview(selection){
    if (selection?.match(/minecraft:(\w+)_concrete/)) {
        return `Set brush color to ${(selection?.match(/minecraft:(\w+)_concrete/) || [])[1].replace("_", ' ')}`;
    } else if (selection === "minecraft:noteblock") {
        return "Render";
    } else if (selection === "minecraft:double_stone_block_slab3") {
        return"Set brush size to Huge"
    } else if (selection === "minecraft:red_sandstone") {
        return "Set brush size to Big"
    } else if (selection === "minecraft:smooth_red_sandstone_stairs") {
        return "Set brush size to Medium"
    } else if (selection === "minecraft:orange_terracotta") {
        return "Set brush size to Small"
    } else if (selection === "minecraft:smooth_quartz_stairs") {
        return "Circle Tool"
    } else if (selection === "minecraft:piston") {
        return "Line Tool"
    } else if (selection === "minecraft:polished_diorite_stairs") {
        return "Rectangle Tool"
    } else if (selection === "minecraft:redstone_block") {
        return "Exit"
    } else if (selection?.match(/minecraft:(\w+)_shulker_box/)) {
        return `Fill canvas to ${(selection?.match(/minecraft:(\w+)_shulker_box/) || [])[1].replace("_", ' ')}`;
    } else {
        return "No tools selected"
    }

}
world.afterEvents.itemUse.subscribe(eventData => {
    item = eventData.itemStack;
    playerUseItem = eventData.source;
    if (item.typeId === "minecraft:brush" && playerUseItem.hasTag('drawer')) {
        function drawCircle(input1, input2) {
            let midPoint = {x:(input2.x + input1.x)*0.5, y:(input2.y + input1.y)*0.5};
            const semiMajorAxis = Math.abs(input1.x - input2.x) / 2;
            const semiMinorAxis = Math.abs(input1.y - input2.y) / 2;
            const tempDraw = system.runInterval(() => {
                outlineDelay = false;
                for (let i = 0; i <= 72; i++) {
                    const coordX = Math.round(midPoint.x + semiMajorAxis * Math.cos((i / 70) * 2 * Math.PI));
                    const coordY = Math.round(midPoint.y + semiMinorAxis * Math.sin((i / 70) * 2 * Math.PI));
                    fakeInput({x:coordX, y:coordY});
                    if (i > 70) {
                        brushProp.tool = "None";
                        system.clearRun(tempDraw);
                    }
                }
            });
            outlineClear();
        };
        function drawRectangle(input1, input2) {
            outlineClear();
            let x1 = clamp(input1.x, 4983, 5014);
            let y1 = clamp(input1.y, 280, 311);
            let x2 = clamp(input2.x, 4983, 5014);
            let y2 = clamp(input2.y, 280, 311);
            if (x1 > x2) {
                [x1, x2] = [x2, x1];
            };
            if (y1 > y2) {
                [y1, y2] = [y2, y1];
            };
            for (let x = x1; x <= x2; x++) {
                fakeInput({x:x, y:y1});
                fakeInput({x:x, y:y2});
            };
            for (let y = y1; y <= y2; y++) {
                fakeInput({x:x1, y:y});
                fakeInput({x:x2, y:y});
            };
        };
        function drawLine(input1, input2, demo) {
            outlineClear()
            let x1 = input1.x;
            let y1 = input1.y;
            let x2 = input2.x;
            let y2 = input2.y;
            const dx = Math.abs(x2 - x1);
            const dy = Math.abs(y2 - y1);
            const sx = x1 < x2 ? 1 : -1;
            const sy = y1 < y2 ? 1 : -1;
            let err = dx - dy;
            while (true) {
                const coordX = clamp(x1, 4983, 5014);
                const coordY = clamp(y1, 280, 311);
                if (coordX && coordY) {
                    if (demo === true) {
                        world.getDimension("overworld").getBlock({x: coordX, y: coordY, z: -13}).setType("minecraft:stone");
                    } else {
                        fakeInput({x:coordX, y:coordY});
                    }
                    
                }
                if (x1 === x2 && y1 === y2) {
                    break;
                }
                const e2 = 2 * err;
                if (e2 > -dy) {
                    err -= dy;
                    x1 += sx;
                }
                if (e2 < dx) {
                    err += dx;
                    y1 += sy;
                }
            }
        };
        function drawSelectionBox(input1, input2) {
            let xCoord1 = clamp(input1.x, 4983, 5014);
            let yCoord1 = clamp(input1.y, 280, 311);
            let xCoord2 = clamp(input2.x, 4983, 5014);
            let yCoord2 = clamp(input2.y, 280, 311);
            outlineClear();
            world.getDimension("overworld").fillBlocks({x:xCoord1, y:yCoord1, z:-13}, {x:xCoord2, y:yCoord1, z:-13}, BlockPermutation.resolve("minecraft:stone"));
            world.getDimension("overworld").fillBlocks({x:xCoord1, y:yCoord1, z:-13}, {x:xCoord1, y:yCoord2, z:-13}, BlockPermutation.resolve("minecraft:stone"));
            world.getDimension("overworld").fillBlocks({x:xCoord2, y:yCoord2, z:-13}, {x:xCoord1, y:yCoord2, z:-13}, BlockPermutation.resolve("minecraft:stone"));
            world.getDimension("overworld").fillBlocks({x:xCoord2, y:yCoord2, z:-13}, {x:xCoord2, y:yCoord1, z:-13}, BlockPermutation.resolve("minecraft:stone"));
        };
        function outlineClear() {
            world.getDimension("overworld").fillBlocks({x:4983, y:279, z:-13}, {x:5014, y:311, z:-13}, BlockPermutation.resolve("minecraft:air"));
        };
        function delaySneakDraw() {
            outlineDelay = false;
            system.runTimeout(() => {
                outlineDelay = true;
            }, 20)
        };
        if (typeof location !== 'undefined' && brushProp.tool === "Circle Tool") {
            const tempDrawBox = system.runInterval(() => {
                outlineDelay = false;
                if (playerUseItem.isSneaking) {
                    selectionList.length = 0;
                    brushProp.tool = "None";
                    outlineClear();
                    delaySneakDraw()
                    system.clearRun(tempDrawBox);
                };
                if (selectionList.length === 1) {
                    drawSelectionBox(selectionList[0], location);
                }
            })
            if (selectionList.length === 0) {
                selectionList[0] = location;
            } else if (selectionList.length === 1) {
                selectionList[1] = location;
                drawCircle(selectionList[0], selectionList[1]);
                selectionList.length = 0;
                system.clearRun(tempDrawBox);
            }
        };
        if (typeof location !== 'undefined' && brushProp.tool === "Line Tool") {
            let tempDrawLine = system.runInterval(() => {
                outlineDelay = false;
                if (playerUseItem.isSneaking) {
                    selectionList.length = 0;
                    brushProp.tool = "None";
                    outlineClear();
                    delaySneakDraw()
                    system.clearRun(tempDrawLine);
                };
                if (selectionList.length === 1) {
                    drawLine(selectionList[0], location, true);
                }
            })
            if (selectionList.length === 0) {
                selectionList[0] = location;
            } else if (selectionList.length === 1) {
                selectionList[1] = location;
                drawLine(selectionList[0], selectionList[1], false);
                selectionList.length = 0;
                brushProp.tool = "None";
                system.clearRun(tempDrawLine);
            }
        };
        if (typeof location !== 'undefined' && brushProp.tool === "Rectangle Tool") {
            let tempDrawRectangle = system.runInterval(() => {
                outlineDelay = false;
                if (playerUseItem.isSneaking) {
                    selectionList.length = 0;
                    brushProp.tool = "None";
                    outlineClear();
                    delaySneakDraw()
                    system.clearRun(tempDrawRectangle);
                };
                if (selectionList.length === 1) {
                    drawSelectionBox(selectionList[0], location, true);
                }
            })
            if (selectionList.length === 0) {
                selectionList[0] = location;
            } else if (selectionList.length === 1) {
                selectionList[1] = location;
                drawRectangle(selectionList[0], selectionList[1], false);
                selectionList.length = 0;
                brushProp.tool = "None";
                system.clearRun(tempDrawRectangle);
            }
        }
        if (selection?.match(/minecraft:(\w+)_wool/) && typeof location !== 'undefined' && brushProp.tool === "None") {
            fakeInput(location)
        }
        if (selection?.match(/minecraft:(\w+)_concrete/)) {
            brushProp.color = (selection?.match(/minecraft:(\w+)_concrete/) || [])[1];
            updateFillTool(brushProp.color);
        } else if (selection === "minecraft:noteblock") {
            render();
        } else if (selection === "minecraft:double_stone_block_slab3") {
            brushProp.size = "Huge"
        } else if (selection === "minecraft:red_sandstone") {
            brushProp.size = "Big"
        } else if (selection === "minecraft:smooth_red_sandstone_stairs") {
            brushProp.size = "Medium"
        } else if (selection === "minecraft:orange_terracotta") {
            brushProp.size = "Small"
        } else if (selection === "minecraft:smooth_quartz_stairs") {
            brushProp.tool = "Circle Tool"
        } else if (selection === "minecraft:piston") {
            brushProp.tool = "Line Tool"
        } else if (selection === "minecraft:polished_diorite_stairs") {
            brushProp.tool = "Rectangle Tool"
        } else if (selection === "minecraft:redstone_block"  && !playerUseItem.isSneaking) {
            playerUseItem.runCommand("title @s subtitle §cYou need to sneak & click to exit");
            playerUseItem.runCommand("title @s title ✅");
        } else if (selection === "minecraft:redstone_block" && playerUseItem.isSneaking) {
            exit = true;
        } else if (selection?.match(/minecraft:(\w+)_shulker_box/)) {
            const color = (selection?.match(/minecraft:(\w+)_shulker_box/) || [])[1];
            world.getDimension("overworld").fillBlocks({x:4983, y:311, z:-14}, {x:5014, y:280, z:-14}, BlockPermutation.resolve(`minecraft:${color}_wool`));
        }
    }
});
system.runInterval(
    () => {
        if (!universalCheck) {checkDrawer()};
        let formatColor = brushProp.color.replace("_", ' ').charAt(0).toUpperCase() + brushProp.color.slice(1).replace("_", ' ').toUpperCase();
        world.getAllPlayers().filter(player => player.hasTag('drawer')).forEach(playerWithTag => {
            let toolDisplay = `${brushProp.tool}-${selectionList.length}`;
            switch (toolDisplay) {
            case 'None-0':
                playerWithTag.onScreenDisplay.setActionBar(`Selected color: ${formatColor} |||| Brush size: ${brushProp.size} \n ${toolPreview(selection)}`)
                //playerWithTag.onScreenDisplay.setActionBar(`${playerWithTag.location.x.toFixed(3)}  ${playerWithTag.location.z.toFixed(3)}`)
                break;
            case 'Circle Tool-0':
                playerWithTag.onScreenDisplay.setActionBar(`Circle Tool: Please choose one point \n Sneak to cancel`)
                break;
            case 'Circle Tool-1':
                playerWithTag.onScreenDisplay.setActionBar(`Circle Tool: Please choose the second point \n Sneak to cancel`)
                break;
            case 'Line Tool-0':
                playerWithTag.onScreenDisplay.setActionBar(`Line Tool: Please choose one point \n Sneak to cancel`)
                break;
            case 'Line Tool-1':
                playerWithTag.onScreenDisplay.setActionBar(`Line Tool: Please choose the second point \n Sneak to cancel`)
                break;
            case 'Rectangle Tool-0':
                playerWithTag.onScreenDisplay.setActionBar(`Rectangle Tool: Please choose one point \n Sneak to cancel`)
                break;
            case 'Rectangle Tool-1':
                playerWithTag.onScreenDisplay.setActionBar(`Rectangle Tool: Please choose the second point \n Sneak to cancel`)
                break;
            }
        });
        if (exit) {
            if (playerPastLocation) {
                world.getDimension(`overworld`).runCommand(`execute as @a[tag=drawer] run tp @s ${playerPastLocation.x} ${playerPastLocation.y} ${playerPastLocation.z}`);
            } else {
                world.getDimension(`overworld`).runCommand(`execute as @a[tag=drawer] run tp @s ${parseInt(world.getDimension("overworld").getBlock({x: 5002, y: 302, z: 14}).getComponent("minecraft:sign")?.getText())} ${parseInt(world.getDimension("overworld").getBlock({x: 5001, y: 302, z: 14}).getComponent("minecraft:sign")?.getText())} ${parseInt(world.getDimension("overworld").getBlock({x: 5000, y: 302, z: 14}).getComponent("minecraft:sign")?.getText())}`);
            }
            removeAll()
            world.getDimension(`overworld`).runCommand(`execute as @a[tag=drawer] run gamemode survival @s`);
            world.getDimension(`overworld`).runCommand(`execute as @a[tag=drawer] run effect @s clear`);
            world.getDimension(`overworld`).runCommand(`execute as @a[tag=drawer] run tag @s remove drawer`);
            exit = false;
            universalCheck = false;
        }
        if (item?.typeId === "minecraft:brush" && playerUseItem?.hasTag('drawer')) {
            try {
                selection = playerUseItem.getBlockFromViewDirection()?.block?.typeId;
                location = playerUseItem.getBlockFromViewDirection()?.block?.location;
            } catch(e) {};
        };
        if ( selection?.match(/minecraft:(\w+)_wool/) && outlineDelay && brushProp.tool === "None" && playerUseItem?.isSneaking && brushProp.color !== "none" && typeof location !== 'undefined' && playerUseItem?.getComponent("minecraft:inventory")?.container?.getItem(playerUseItem?.selectedSlot)?.typeId === "minecraft:brush") {
            fakeInput(location);
        };
    },
);